import java.awt.Frame;
import java.awt.Panel;
import java.awt.Button;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.awt.Choice;
import java.util.Vector;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.io.IOException;

public class PacketViewer extends Frame implements ActionListener, ItemListener {

	public static final String IP_SENDER = "Sender";
	public static final String IP_DESTINATION = "Destination";


	protected Grapher graImager = null;
	protected String strCurStats;
	protected PacketParser ppPackets = null;
	protected MenuBar mnbMenu = null;
	protected Menu mnuFile = null;
	protected Menu mnuConfig = null;
	protected Frame fraPopUp = null;
	protected Panel pnlIPs = null;
	protected GraphPanel pnlGraph = null;
	protected Panel pnlStats = null;
	protected int iMinimumPackets = 10;
	protected String strIPType = IP_SENDER;
	protected CVPacketList[] cvplaAddressList;

	/**
	 ***********************************************************************
	 *
	 *
	 */
	public PacketViewer(String strFilename) {

		super("CommView based packet display tool");

		if(!(strFilename == null))
			ppPackets = new PacketParser(strFilename);
		strCurStats = new String();
		setupDisplay(strCurStats);

	}//end CVFrame(CVPacket[])


	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected void setupDisplay(String strStats) {

System.out.println(ppPackets);

		System.out.println("setupDisplay");

		GridBagConstraints gbc = new GridBagConstraints();

		this.removeAll();
		this.setLayout(new GridBagLayout());
		this.setSize(1000,600);
		this.setMenuBar(getmnbMenu());

		gbc.insets = new Insets(10,10,10,10);
		gbc.gridwidth = 4; gbc.gridheight = 2;
		gbc.weightx = 4; gbc.weighty = 2;
		gbc.fill = GridBagConstraints.BOTH;
		this.add(getpnlGraph(),gbc);

		gbc.weightx = 1; gbc.weighty = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
 	    gbc.gridheight = 1;
 	    gbc.fill = GridBagConstraints.HORIZONTAL;
		this.add(getpnlIPs(),gbc);
		this.add(getpnlStats(strStats),gbc);
		this.validate();


	}//setupDisplay

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Menu getmnuFile() {

		if(mnuFile == null) {

			mnuFile = new Menu("File");

			MenuItem miOpen = new MenuItem("Open");
			miOpen.addActionListener(this);
			mnuFile.add(miOpen);

			MenuItem miExit = new MenuItem("Exit");
			miExit.addActionListener(this);
			mnuFile.add(miExit);

		}//end if
		return mnuFile;
	}//end getmnuFile

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Menu getmnuConfig() {

		if(mnuConfig==null) {
			mnuConfig = new Menu("Configuration");

			MenuItem miIPFilter = new MenuItem("IP Filtering");
			miIPFilter.addActionListener(this);
			mnuConfig.add(miIPFilter);

			MenuItem miIPType = new MenuItem("IP Type");
			miIPType.addActionListener(this);
			mnuConfig.add(miIPType);

		}//end if

		return mnuConfig;
	}//end getmnuConfig

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected MenuBar getmnbMenu() {
		if(mnbMenu == null) {

			mnbMenu = new MenuBar();
			mnbMenu.add(getmnuFile());
			mnbMenu.add(getmnuConfig());
		}//if

		return mnbMenu;
	}//getmnbMenuBar

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Vector getIPs(boolean bSender, int iLowerPacketLimit) {

		Vector vecList = new Vector();
		Vector vecReturn = new Vector();
		CVPacket cvpCurPacket;
		CVPacket[] cvpaPackets;
		String strCurIP;
		int j, count;
		int i;

		if(ppPackets!= null) {
			cvpaPackets = ppPackets.cvpaList;

			if(bSender) {
				for(i=0;i<cvpaPackets.length;i++) {
					cvpCurPacket = cvpaPackets[i];
					vecList.addElement(cvpCurPacket.strSenderIP);
				}//end for
			} else {
				for(i=0;i<cvpaPackets.length;i++) {
					cvpCurPacket = cvpaPackets[i];
					vecList.addElement(cvpCurPacket.strDestinationIP);
				}//end for
			}//end else

			i=0;
			while(i<vecList.size()) {
				count = 0;
				strCurIP = (String)vecList.elementAt(i);
				for(j=i;j<vecList.size();j++) {
					if(strCurIP.equals((String)vecList.elementAt(j))) {
						count++;
					}//end if
				}//end for
				if( (count>=iLowerPacketLimit) && !(vecReturn.contains(strCurIP))) {
					vecReturn.addElement(strCurIP);
				}//end if
				i++;
			}//end while
			return vecReturn;
		} else return null;
	}//end getIPs

	/**
	 ***********************************************************************
	 *
	 *
	 */

	protected Panel getpnlIPs() {

		if(pnlIPs == null) {

			Vector vecIPList;
			GridBagConstraints gbc;
			String[] straColorList = {"none","black","blue","cyan","dark gray","gray","green",
									  "light gray","magenta","orange","pink","red",
									  "white","yellow"};
			String strCurIP;
			Choice choCurBox = null;
			int i;

			pnlIPs = new Panel();

			vecIPList = getIPs(strIPType.equals(IP_SENDER),iMinimumPackets);

			if(vecIPList!=null) {
				cvplaAddressList = new CVPacketList[vecIPList.size()];
				pnlIPs.setLayout(new GridLayout(vecIPList.size()+1,2,1,1));
				pnlIPs.add(new Label("IP address"));
				pnlIPs.add(new Label("Color"));

				for(i=0;i<vecIPList.size();i++) {
					strCurIP = (String)vecIPList.elementAt(i);
					pnlIPs.add(new Label(strCurIP));
					choCurBox = new Choice();
					for(int j=0;j<straColorList.length;j++)
						choCurBox.add(straColorList[j]);
					choCurBox.setName(strCurIP);
 					choCurBox.addItemListener(this);
					cvplaAddressList[i] = new CVPacketList(ppPackets.cvpaList,
														   strIPType.equals(IP_SENDER),
														   strCurIP);
					pnlIPs.add(choCurBox);
				}//end for

			} else {
				System.out.println("Else");
				pnlIPs.setLayout(new GridLayout(1,2,5,5));
				pnlIPs.add(new Label("IP address"));
				pnlIPs.add(new Label("Color"));
			}//end else


		}//end if
		return pnlIPs;

	}//end getpnlIPs

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Panel getpnlGraph() {

		pnlGraph = new GraphPanel();
		pnlGraph.setLayout(new GridBagLayout());
		graImager = new Grapher();
		if(cvplaAddressList == null) {
			pnlGraph.add(new Label("Graph"));
			return pnlGraph;
		}
		for(int i=0;i<cvplaAddressList.length;i++) {
			if(cvplaAddressList[i].cSelectedColor != null)
				graImager.addCurve(cvplaAddressList[i].cvpaPackets,
								   cvplaAddressList[i].strThisIPAddr,
								   cvplaAddressList[i].strThisIPAddr,
								   cvplaAddressList[i].cSelectedColor);
		}//end for
		try {
			pnlGraph.setGraph(graImager.renderGraph(strCurStats,500,300));
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		return pnlGraph;
	}//end

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Panel getpnlStats(String strIPName) {

			GridBagConstraints gbc = new GridBagConstraints();
			pnlStats = new Panel();
			pnlStats.setLayout(new GridBagLayout());

			gbc.insets = new Insets(0,10,0,10);
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			pnlStats.add(new Label("Statistics for "+strIPName),gbc);

			int index = findPacketList(strIPName);
			if(index==-1)
				return pnlStats;
			CVPacketList cvplSelectedIP = cvplaAddressList[index];
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			pnlStats.add(new Label("Total Packets: "+cvplSelectedIP.cvpaPackets.length),gbc);
			gbc.gridwidth = 1;
			pnlStats.add(new Label("Large Packet:"+cvplSelectedIP.iLargestPacket+" bytes"),gbc);
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			pnlStats.add(new Label("Small Packet:" +cvplSelectedIP.iSmallestPacket+" bytes"),gbc);
			gbc.gridwidth = 1;
			pnlStats.add(new Label("Mean Size:"+ cvplSelectedIP.dMeanSize+" bytes"),gbc);
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			pnlStats.add(new Label("Standard Dev Size:"+ cvplSelectedIP.dStdDevSize+" bytes"),gbc);
			gbc.gridwidth = 1;
			pnlStats.add(new Label("Mean Time:"+ cvplSelectedIP.dMeanTimeBetweenPackets+"s"),gbc);
			gbc.gridwidth = GridBagConstraints.REMAINDER;
			pnlStats.add(new Label("Standard Dev Time:"+ cvplSelectedIP.dStdDevTimeBetweenPackets+"s"),gbc);


		return pnlStats;
	}//end

	/**
	 ***********************************************************************
	 *
	 *
	 */
	public int findPacketList(String strCurString) {

		if(cvplaAddressList == null)
			return -1;
		for(int i=0;i<cvplaAddressList.length;i++)
			if(strCurString.equals(cvplaAddressList[i].strThisIPAddr))
				return i;
		return -1;

	}//end findPacketList(strCurString)
	/**
	 ***********************************************************************
	 *
	 *
	 */
	public void itemStateChanged(ItemEvent ie) {

		System.out.println("IE:"+ie);
		GridBagConstraints gbc = new GridBagConstraints();
		Choice choSelectedIP = (Choice)ie.getItemSelectable();
		strCurStats = choSelectedIP.getName();
		this.remove(pnlStats);
		int index = findPacketList(strCurStats);
		cvplaAddressList[index].cSelectedColor = getColor(choSelectedIP.getSelectedItem());
		setupDisplay(strCurStats);

	}//itemStateChanged

	/**
	 ***********************************************************************
	 *
	 *
	 */
	public void actionPerformed(ActionEvent ae) {

		String strEventText = ae.getActionCommand();

		System.out.println(strEventText);

		if(strEventText.equals("Exit"))
			System.exit(0);
		if(strEventText.equals("Open")) {
			JFileChooser fc = new JFileChooser("c:\\progra~1\\Commview\\logs\\");
			int returnVal = fc.showOpenDialog(this);
			String strSelectedFile;
			System.out.println("returnval:"+returnVal);
			if(returnVal != JFileChooser.CANCEL_OPTION) {
				strSelectedFile = fc.getSelectedFile().getAbsolutePath();
				ppPackets = new PacketParser(strSelectedFile);
				this.remove(pnlIPs);
				pnlIPs = null;
				setupDisplay(strCurStats);
			}//end if
		}//end if
		if(strEventText.equals("IP Type")) {
			Object[] possibleValues = { IP_SENDER, IP_DESTINATION };
		 	strIPType = (String)JOptionPane.showInputDialog(null,
		 												    "Choose one",
		 												    "Input",
		 												    JOptionPane.INFORMATION_MESSAGE,
		 												    null,
		 												    possibleValues,
		 												    possibleValues[0]);

		}//end if
		if(strEventText.equals("IP Filtering")) {
			String strInputValue;
			StringBuffer strbDisplayString = new StringBuffer();

			strbDisplayString.append("Enter the minimum number of packets that must be ");
			if(strIPType.equals(IP_SENDER))
				strbDisplayString.append("sent ");
			else strbDisplayString.append("received ");
			strbDisplayString.append("by and IP address to have it graphed below.");

			strInputValue = JOptionPane.showInputDialog(strbDisplayString.toString());
			try {
				iMinimumPackets = new Integer(strInputValue).intValue();
			} catch(NumberFormatException nfe) {
				strInputValue = JOptionPane.showInputDialog("Please enter a whole number.");
			}//end catch
		}//end if
	}//end actionPerformed

	/**
	  **************************************************
	  *
	  *
	  */
	protected Color getColor(String strColor) {

		if(strColor.equals("black"))
			return Color.black;
		if(strColor.equals("blue"))
			return Color.blue;
		if(strColor.equals("cyan"))
			return Color.cyan;
		if(strColor.equals("dark gray"))
			return Color.darkGray;
		if(strColor.equals("gray"))
			return Color.gray;
		if(strColor.equals("green"))
			return Color.green;
		if(strColor.equals("light gray"))
			return Color.lightGray;
		if(strColor.equals("magenta"))
			return Color.magenta;
		if(strColor.equals("orange"))
			return Color.orange;
		if(strColor.equals("pink"))
			return Color.pink;
		if(strColor.equals("red"))
			return Color.red;
		if(strColor.equals("white"))
			return Color.white;
		if(strColor.equals("yellow"))
			return Color.yellow;
		return null;

	}

	/**
	  **************************************************
	  *
	  *
	  */
	public static void main(String[] straArgs) {

		PacketViewer pvInstance;
try {
		if(straArgs.length==1) {
			pvInstance = new PacketViewer(straArgs[0]);
		} else if(straArgs.length==2) {
				pvInstance = new PacketViewer(straArgs[0]+" "+straArgs[1]);
			 } else {
				 pvInstance = new PacketViewer(null);
			 }//end else
		pvInstance.show();
} catch (Exception e) {
	e.printStackTrace();
}
	}//end main(String[])

}//end CVFrame