import java.io.IOException;


/**
 **************************************************
 **************************************************
 * The internal representation of a Commview Packet
 * @author Josh Winslow
 */
public class CVPacket extends Packet {

	//Various offsets to important data in a ccv file
	public static final int DIRECTION_LOCATION = 1;
	public static final int TIME_START = 4;
	public static final int TIME_END = 18;

	public int iDirection;

    /**
     **************************************************
     * Run of the mill constructor
     *
     */
	public CVPacket(String strCVString) throws IOException {

		Integer intTemp;
		String strTemp;

		if( !(strCVString.startsWith("#")) && !(strCVString.endsWith("#")) )
			throw new IOException("Not a Commview packet.");
		intTemp = new Integer(strCVString.substring(DIRECTION_LOCATION,DIRECTION_LOCATION+1));
		iDirection = intTemp.intValue();

		ptTime = computeTime(strCVString.substring(TIME_START, TIME_END));
		strSenderIP = parseIP(strCVString.substring(72,80));
		strDestinationIP = parseIP(strCVString.substring(80,88));
		strTemp = strCVString.substring( 104, (strCVString.length()-1) );
		iPacketSize = strCVString.length()-TIME_END-1;

	}//end CVPacket(String)

    /**
     **************************************************
     * Given the odd hex based time format that Commview uses
     * this function internalizes the time
     *
     */
	protected PacketTime computeTime(String strTime) throws IOException {

		int iHour;
		int iMinute;
		int iSecond;
		int iMillisecond;
		PacketTime ptTemp;

		iMinute = Integer.parseInt(strTime.substring(0,2),16);
		iHour = Integer.parseInt(strTime.substring(4,6),16);
		iMillisecond = Integer.parseInt(strTime.substring(6,10),16);
		iSecond = Integer.parseInt(strTime.substring(12,14),16);
		ptTemp = new PacketTime(iHour,iMinute,iSecond,iMillisecond);

		return ptTemp;
	}//end computeTime(String)

}//end CVPacket


