import java.io.FileReader;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

/**
 **************************************************
 **************************************************
 * This file does most of the upper level management
 * of the reading and output of the packet data.
 * It currently outputs several types of files for
 * each IP address:
 * @author Josh Winslow
 *
 */
public class PacketWriter {

	public static final int MIN_COUNT = 100;

	Packet[] pktaPacketLists;
	String strOutputPath;
	PacketParser ppParser;
	File[] faFileList;
	Vector vecSenderIPList;

    /**
     **************************************************
 	 * Constructor, requires path to file with list of packet
     * files.
     */
	public PacketWriter(String strPath) throws IOException {

		if(strPath==null)
			throw new IOException("A filename must be specified.");

		File fPath;
		String strFile;

		fPath= new File(strPath);
 		if(!fPath.isDirectory())
			throw new IOException("IOE");
		faFileList = fPath.listFiles();
		Utils.sortByName(faFileList);

	}//end PacketWriter(String)

    /**
     **************************************************
 	 * If you wish to add support for a different packetParser,
 	 * you must add a try-catch here.
     */
	protected PacketParser getParser(String strFilename) {

		PacketParser ppReturn = null;

		try {
			ppReturn = new CVPacketParser(strFilename);
		} catch (IOException ioe) {
		}
		return ppReturn;
	}//end getParser(strFilename)

	/**
     **************************************************
	 * Reads in from each of the files and merges the Packet
	 * arrays into one array.
	 */
	protected void parseFiles() {

		Vector vecPacketList = new Vector();
		Packet[] pktaPackets;
		int iTotalLength = 0;

		//read in each file, trying all packet parsers for each file
		for(int i=0;i<faFileList.length;i++) {
			ppParser = getParser(faFileList[i].getAbsolutePath());
			pktaPackets = ppParser.pktaList;
			vecPacketList.addElement(pktaPackets);
			iTotalLength += pktaPackets.length;
		}//end for

		pktaPacketLists = new Packet[iTotalLength];
		iTotalLength = 0;

		//merge the packet arrays
		for(int i=0;i<vecPacketList.size();i++) {
			pktaPackets = (Packet[])vecPacketList.elementAt(i);
			for(int j=0;j<pktaPackets.length;j++) {
				pktaPacketLists[iTotalLength] = pktaPackets[j];
				iTotalLength++;
			}//end for
		}//end for
	}//end parseFiles

	/**
     **************************************************
     * Makes a list of all the sender IP addresses in
     * the merged packet file list
     */
	private void findIPs() {

		vecSenderIPList = new Vector();

		for(int i=0;i<pktaPacketLists.length;i++) {
			//if its not in the list, add it
			if(!vecSenderIPList.contains(pktaPacketLists[i].strSenderIP)) {
				vecSenderIPList.addElement(pktaPacketLists[i].strSenderIP);
			}//end if
		}//end for
	}//end findIPs

	/**
     **************************************************
     * Outputs the Vector of DataCounts to the specifed file
     */
	public void outputPairs(Vector vecList, String strFilename) throws IOException {

		FileWriter fwOutput;
		DataCount dcCur;

		fwOutput = new FileWriter(strFilename);
		for(int i=0;i<vecList.size();i++) {
			dcCur = (DataCount)vecList.elementAt(i);
			fwOutput.write(dcCur.toString()+"\n");
			fwOutput.flush();

		}//end for
	}//end outputPairs(Vector,String)

	/**
     **************************************************
     * The same as outputPairs, but the list is sorted in
     * decending order by the count attribute of DataCount.
     */
	public void outputPairsDecending(Vector vecList, String strFilename) throws IOException {

		DataCount dc1, dc2;
		//slow sort cause the lists aren't all that big
		for(int i=0;i<vecList.size();i++) {
			dc1 = (DataCount)vecList.elementAt(i);
			for(int j=i+1;j<vecList.size();j++) {
				dc2 = (DataCount)vecList.elementAt(j);
				if(dc1.count<dc2.count) {
					vecList.setElementAt(dc2,i);
					vecList.setElementAt(dc1,j);
					dc1 = dc2;
				}//end if
			}//end for
		}//end for
		outputPairs(vecList,strFilename);
	}//end outputPairsDecending(Vector, String)

	/**
     **************************************************
	 * Writes the size, and the time elapsed between the
	 * first packet sent and the current one.
	 */
	public void outputPacketsNoIP(PacketList pktlList, String strFilename)   throws IOException {

		FileWriter fwOutput;
		Packet[] pktaList = pktlList.pktaPackets;


		fwOutput = new FileWriter(strFilename);
		for(int i=0;i<pktaList.length;i++) {
			fwOutput.write(new Double(pktaList[i].getTime()).toString()+",");
			fwOutput.write(pktaList[i].iPacketSize+"\n");
			fwOutput.flush();
		}//end for
	}//end outputPackets

	/**
     **************************************************
	 * Writes the sender and recipent IP addresses, the size, and
	 * the time elapsed between the first packet sent and the current
	 * one.
	 */
	public void outputPackets(PacketList pktlList, String strFilename)   throws IOException {

		FileWriter fwOutput;
		Packet[] pktaList = pktlList.pktaPackets;

		fwOutput = new FileWriter(strFilename);
		for(int i=0;i<pktaList.length;i++) {
			fwOutput.write(pktaList[i].strSenderIP+",");
			fwOutput.write(pktaList[i].strDestinationIP+",");
			fwOutput.write(pktaList[i].iPacketSize+",");
			fwOutput.write(new Double(pktaList[i].getTime()).toString()+"\n");
			fwOutput.flush();
		}//end for
	}//end outputPackets

	public void outputByBandwidth() throws IOException {

		PacketList pktlCur;
		Packet[] pktaList;
		int[] bandwidth;
		int totalBytes = 0;
		Integer iCur;
		Vector vecBlah;

		String strIP;

		for(int i=0;i<vecSenderIPList.size();i++) {

			vecBlah = new Vector();

			strIP = (String)vecSenderIPList.elementAt(i);

			//filter by IP
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			pktaList = pktlCur.pktaPackets;
			if(pktaList.length>MIN_COUNT) {

				bandwidth = new int[(int)pktlCur.dTotalTime+1];
				totalBytes = 0;
				for(int j=0;j<bandwidth.length;j++) {
					bandwidth[j]=0;
				}
				for(int j=0;j<bandwidth.length;j++) {
					for(int k=0;k<pktaList.length;k++) {
						if((pktaList[k].getTime()-j>=0) && (pktaList[k].getTime()-j<1))
							bandwidth[j]+=pktaList[k].iPacketSize;
					}//end for
				}//end for
				for(int j=0;j<bandwidth.length;j++) {
				//	bandwidth[j]-=totalBytes;
				//	totalBytes+=bandwidth[j];
					iCur = new Integer(j);
					vecBlah.addElement(new DataCount(iCur,bandwidth[j]));
				}
				outputPairs(vecBlah,strOutputPath+strIP+"bndwth");
			}
		}//end for
	}//end outputByBandwidth

	/**
     **************************************************
     * Outputs just the size and time delta of each packet.
     */
	public void outputByPacket() throws IOException {

		String strIP;
		PacketList pktlCur;

		for(int i=0;i<vecSenderIPList.size();i++) {
			strIP = (String)vecSenderIPList.elementAt(i);
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			if(pktlCur.pktaPackets.length>MIN_COUNT)
				outputPacketsNoIP(pktlCur,strOutputPath+strIP+"NoIP");
		}//end for
	}//end

	/**
     **************************************************
     * This function calculates the number of times each
     * packet size is found in the traces and then outputs
     * them in decending order.
     */
	public void outputBySize() throws IOException {

		Vector  vecList = new Vector();
		String strIP;
		PacketList pktlCur;
		Packet[] pktaList;
		DataCount dcCur;
		boolean bFound;
		Integer iSize;

		for(int i=0;i<vecSenderIPList.size();i++) {
			strIP = (String)vecSenderIPList.elementAt(i);

			//filter by IP
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			pktaList = pktlCur.pktaPackets;

			if(pktaList.length>MIN_COUNT) {
				for(int j=0;j<pktaList.length;j++) {
					iSize = new Integer(pktaList[j].iPacketSize);
					bFound = false;
					for(int k=0;k<vecList.size();k++) {
						dcCur = (DataCount)vecList.elementAt(k);
						if(dcCur.incrOnEqual(iSize)) {
							bFound=true;
						}
					}//end for
					if(!bFound)
						vecList.addElement(new DataCount(iSize));
				}//end for
				outputPairsDecending(vecList, strOutputPath+strIP+"Size");
			}//end if
		}//end for
	}//end outputBySize

	/**
     **************************************************
     * Writes out Sender IP, Destination IP, Size, and
     * Time Delta on one line per packet.
     */
	public void outputByIP() throws IOException {

		String strIP;
		PacketList pktlCur;

		for(int i=0;i<vecSenderIPList.size();i++) {
			strIP = (String)vecSenderIPList.elementAt(i);
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			outputPackets(pktlCur,strOutputPath+strIP);
		}//end for
	}//end

	/**
     **************************************************
     * Converts the size from total packet numbers
     * IE 300 of 500 of size 130 to a percentage .6 of 130
     */
	public void outputBySizePct() throws IOException {

		Vector  vecList= new Vector();
		String strIP;
		PacketList pktlCur;
		Packet[] pktaList;
		DataCount dcCur;
		boolean bFound;
		Integer iSize;
		int cnt=0;


		for(int i=0;i<vecSenderIPList.size();i++) {
			cnt = 0;
			strIP = (String)vecSenderIPList.elementAt(i);
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			pktaList = pktlCur.pktaPackets;
			for(int j=0;j<pktaList.length;j++) {
				iSize = new Integer(pktaList[j].iPacketSize);
				bFound = false;
				for(int k=0;k<vecList.size();k++) {
					dcCur = (DataCount)vecList.elementAt(k);
					if(dcCur.incrOnEqual(iSize))
						bFound=true;
				}//end for
				if(!bFound)
					vecList.addElement(new DataCount(iSize));
				cnt++;
			}//end for
			for(int j=0;j<vecList.size();j++) {
				dcCur = (DataCount)vecList.elementAt(j);
				dcCur.genPct(cnt);
				dcCur.pctFlag = true;
			}

			outputPairs(vecList, strOutputPath+strIP+"SizePct");
			vecList = new Vector();

		}//end for
	}

	/**
     **************************************************
     *
     */
	public void outputByTimeDelta() throws IOException {

		Vector  vecList= new Vector();
		String strIP;
		PacketList pktlCur;
		Packet[] pktaList;
		DataCount dcCur;
		boolean bFound;
		Double dDelta;

		for(int i=0;i<vecSenderIPList.size();i++) {
			strIP = (String)vecSenderIPList.elementAt(i);

			//filter by IP
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			pktaList = pktlCur.pktaPackets;

			if(pktaList.length>MIN_COUNT) {
				for(int j=1;j<pktaList.length;j++) {
					dDelta = new Double( ((pktaList[j].getTime()) - (pktaList[j-1].getTime())));

					//Out of order packet protection
					if(dDelta.doubleValue()<0)
						dDelta = new Double(0.0);

					bFound = false;
					for(int k=0;k<vecList.size();k++) {
						dcCur = (DataCount)vecList.elementAt(k);
						if(dcCur.incrOnEqual(dDelta)) {
							bFound=true;
						}
					}//end for
					if(!bFound)
						vecList.addElement(new DataCount(dDelta));
				}//end for
				outputPairsDecending(vecList, strOutputPath+strIP+"delta");
			}//end if
			vecList = new Vector();
		}//end for
	}//end outputByDelta
	/**
     **************************************************
     * Converts the time delta from total packet numbers
     * IE 300 of 500 at time 0.3 to a percentage .6 at 0.3
     */
	public void outputByDeltaPct() throws IOException {

		Vector  vecList= new Vector();
		String strIP;
		PacketList pktlCur;
		Packet[] pktaList;
		DataCount dcCur;
		boolean bFound;
		Double dDelta;
		int cnt=0;


		for(int i=0;i<vecSenderIPList.size();i++) {
			cnt = 0;
			strIP = (String)vecSenderIPList.elementAt(i);
			pktlCur = new PacketList(pktaPacketLists,true,strIP);
			pktaList = pktlCur.pktaPackets;
			for(int j=1;j<pktaList.length;j++) {
				dDelta = new Double(pktaList[j].getTime()-pktaList[j-1].getTime());
				bFound = false;
				for(int k=0;k<vecList.size();k++) {
					dcCur = (DataCount)vecList.elementAt(k);
					if(dcCur.incrOnEqual(dDelta))
						bFound=true;
				}//end for
				if(!bFound)
					vecList.addElement(new DataCount(dDelta));
				cnt++;
			}//end for
			for(int j=0;j<vecList.size();j++) {
				dcCur = (DataCount)vecList.elementAt(j);
				dcCur.genPct(cnt);
				dcCur.pctFlag = true;
			}
			outputPairs(vecList, strOutputPath+strIP+"deltapct");
			vecList = new Vector();
		}//end for
	}//end outputByDelta

	/**
     **************************************************
     * The entry point to the class.  Any new output types need
     * to be added here and given an option.
     */
	public static void main(String[] straArgs) {

		if( (straArgs.length<2) || (straArgs.length>10) ) {
			System.out.println("The usage is:\njava PacketWriter OPTIONS INPUT_PATH OUTPUT_PATH\n"+
						   	   "where OPTIONS are any of -IP -T -SP -S -TP -B\n"+
						   	   "which output by IP, Size, Size by Percentage, Time Delta, and Time Delta by Percentage, and Bandwidth per Second repsectively.\n"+
 							   "Where INPUT_PATH is the name of a directory that contains all the packet files.\n"+
							   "and where OUTPUT_PATH is the name of the directory in which to output the packet data."+
							   "If no options are specified, -T -P -S -B will be assumed.");
			System.exit(0);
		}//end if
		try {
			PacketWriter pdInstance = new PacketWriter(straArgs[straArgs.length-2]);
			pdInstance.strOutputPath = straArgs[straArgs.length-1];
			pdInstance.parseFiles();
			pdInstance.findIPs();
			if(straArgs.length==2) {
				pdInstance.outputByTimeDelta();
				pdInstance.outputByPacket();
				pdInstance.outputBySize();
				pdInstance.outputByBandwidth();
			}
			for(int i=0;i<straArgs.length-2;i++) {
				if(straArgs[i].equals("-IP"))
					pdInstance.outputByIP();
				if(straArgs[i].equals("-S"))
					pdInstance.outputBySize();
				if(straArgs[i].equals("-SP"))
					pdInstance.outputBySizePct();
				if(straArgs[i].equals("-T"))
					pdInstance.outputByTimeDelta();
				if(straArgs[i].equals("-TP"))
					pdInstance.outputByDeltaPct();
				if(straArgs[i].equals("-P"))
					pdInstance.outputByPacket();
				if(straArgs[i].equals("-B"))
					pdInstance.outputByBandwidth();
			}//end for

		} catch(IOException ioe) {
			System.out.println("Error!");
			ioe.printStackTrace();
		}//end catch
	}//end main
}//end PacketWriter